package com.alibaba.fastjson.parser.deserializer;

import java.io.StringWriter;
import java.lang.reflect.Type;

import com.alibaba.fastjson.JSONException;
import com.alibaba.fastjson.parser.DefaultExtJSONParser;

public class AppendableDeserializer implements ObjectDeserializer {

    public final static AppendableDeserializer instance = new AppendableDeserializer();

    @SuppressWarnings("unchecked")
    public <T> T deserialze(DefaultExtJSONParser parser, Type type) {
        Object value = parser.parse();

        if (value == null) {
            return null;
        }

        if (type == Appendable.class || type == StringBuilder.class) {
            StringBuilder buf = new StringBuilder();
            buf.append(value.toString());
            return (T) buf;
        }

        if (type == StringBuffer.class) {
            StringBuffer buf = new StringBuffer();
            buf.append(value.toString());
            return (T) buf;
        }

        if (type == StringWriter.class) {
            StringWriter buf = new StringWriter();
            buf.append(value.toString());
            return (T) buf;
        }

        try {
            Class<?> clazz = (Class<?>) type;
            Appendable inst = (Appendable) clazz.newInstance();
            inst.append(value.toString());
            
            return (T) inst;
        } catch (Exception e) {
            throw new JSONException(e.getMessage(), e);
        }
    }

}
