package com.alibaba.fastjson.parser.deserializer;

import java.lang.reflect.Method;
import java.lang.reflect.Type;
import java.util.HashMap;
import java.util.Map;

import com.alibaba.fastjson.JSONException;
import com.alibaba.fastjson.parser.DefaultExtJSONParser;

public class EnumDeserializer implements ObjectDeserializer {

    public final static EnumDeserializer instance = new EnumDeserializer();

    @SuppressWarnings({ "unchecked", "rawtypes" })
    public <T> T deserialze(DefaultExtJSONParser parser, Type type) {
        try {
            Class<T> clazz = (Class<T>) type;
            
            Object value = parser.parse();

            if (value instanceof String) {
                String strVal = (String) value;
                if (strVal.length() == 0) {
                    return (T) null;
                }
                return (T) Enum.valueOf((Class<Enum>) clazz, strVal);
            } else if (value instanceof Integer) {
                initEnum(parser, (Class<Enum>) clazz);
                Map<Integer, Enum> items = parser.getMapping().getEnumValues().get(clazz);

                T e = (T) items.get(value);
                if (e == null) {
                    throw new JSONException("parse enum " + clazz.getName() + " error, value : " + value);
                }
                return e;
            } else {
                throw new JSONException("parse enum " + clazz.getName() + " error, value : " + value);
            }
        } catch (JSONException e) {
            throw e;
        } catch (Throwable e) {
            throw new JSONException(e.getMessage(), e);
        }
    }

    @SuppressWarnings("rawtypes")
    public void initEnum(DefaultExtJSONParser parser, Class<? extends Enum> clazz) throws Exception {
        Map<Integer, Enum> items = parser.getMapping().getEnumValues().get(clazz);

        if (items == null) {
            items = new HashMap<Integer, Enum>();

            Method method = clazz.getMethod("values");
            Object[] values = (Object[]) method.invoke(null);
            for (Object value : values) {
                Enum e = (Enum) value;
                items.put(e.ordinal(), e);
            }

            parser.getMapping().getEnumValues().putIfAbsent(clazz, items);
            items = parser.getMapping().getEnumValues().get(clazz);
        }
    }
}
