package com.alibaba.fastjson.serializer;

import java.io.IOException;

import javax.management.openmbean.CompositeData;
import javax.management.openmbean.CompositeType;

public class CompositeDataSerializer implements ObjectSerializer {

    public final static CompositeDataSerializer instance = new CompositeDataSerializer();

    public void write(JSONSerializer serializer, Object object) throws IOException {
        CompositeData data = (CompositeData) object;

        SerializeWriter out = serializer.getWrier();

        out.write('{');
        boolean first = true;

        CompositeType type = data.getCompositeType();

        for (Object key : type.keySet()) {
            Object value = data.get((String) key);

            if (value == null) {
                if (!serializer.isEnabled(SerializerFeature.WriteMapNullValue)) {
                    continue;
                }
            }

            if (!first) {
                out.write(',');
            }

            if (serializer.isEnabled(SerializerFeature.UseSingleQuotes)) {
                if (serializer.isEnabled(SerializerFeature.QuoteFieldNames)) {
                    out.writeKeyWithSingleQuote((String) key);
                } else {
                    out.writeKeyWithSingleQuoteIfHasSpecial((String) key);
                }
            } else {
                if (serializer.isEnabled(SerializerFeature.QuoteFieldNames)) {
                    out.writeKeyWithDoubleQuote((String) key);
                } else {
                    out.writeKeyWithDoubleQuoteIfHasSpecial((String) key);
                }
            }

            serializer.write(value);

            first = false;
        }

        out.write('}');
    }

}
